package xyz.matj1.bp;

import mcdr.objects.impl.ObjectMotionWordSoftAssignment;
import messif.objects.LocalAbstractObject;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.XYPlot;
import xyz.matj1.bp.enums.MocapType;
import xyz.matj1.bp.experiments.KnnPrecision;
import xyz.matj1.bp.experiments.PairwiseCorrelation;
import xyz.matj1.bp.themes.SmoothXyGraphTheme;
import xyz.matj1.bp.types.Vector2;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.time.Duration;
import java.time.Instant;
import java.util.Optional;

import static xyz.matj1.bp.BPBagar2.loadData;
import static xyz.matj1.bp.enums.LengthRelation.QUOTIENT;


/**
 * A class for the testing of how sequences of
 * motion words of similar motions of different lengths can be compared
 *
 * @author Matěj Bagar
 */
public class LengthTesting {
	public static void main(String[] args) throws IOException{
		ObjectMotionWordSoftAssignment.maxPartsToMatch = 6;

		interface ShowGraph {
			void show(String dataPath, Class<? extends LocalAbstractObject> dataClass, String outputPath) throws IOException;
		}

		ShowGraph showQuotientGraph = (dataPath, dataClass, outputDirPath) -> {
			var timer = Instant.now();
			PairwiseCorrelation.show(
				loadData(dataPath, dataClass),
				QUOTIENT,
				new File(outputDirPath)
			);
			System.err.printf(
				"Time to compute pairwise correlelations for data class %s: %d s%n",
				dataClass.getSimpleName(),
				Duration.between(timer, Instant.now()).toSeconds()
			);
		};

		ShowGraph showKnnLengthGraph = (dataPath, dataClass, outputDirPath) -> {
			var timer = Instant.now();
			KnnPrecision.show(
				loadData(dataPath, dataClass),
				new File(outputDirPath)
			);
			System.err.printf(
				"Time to compute KNN for data class %s: %d s%n",
				dataClass.getSimpleName(),
				Duration.between(timer, Instant.now()).toSeconds()
			);
		};

		// pairwise correlation tests (to graphs)
		if (false)
		{
			for (var type : MocapType.values()) {
				showQuotientGraph.show(
					dataFileNames(type),
					type.dataClass(),
					"output/" + type.adjective() + " quotient pairwise graphs"
				);
			}
		}

		// pairwise tests put to a single dataset and smoothed (to graphs)
		if (false)
		{
			for (var type : MocapType.values()) {
				Utils.writeXYGraphAsPdf(
					PairwiseCorrelation.showCategory(
						PairwiseCorrelation.smooth(
							PairwiseCorrelation.compute(
								loadData(dataFileNames(type), type.dataClass()),
								QUOTIENT
							),
							0.01d,
							.5d
						),
						QUOTIENT,
						Optional.empty()
					),
					new File("output/smoothed quotient pairwise graphs", type.noun() + ".pdf"),
					new Vector2<>(512d, 512d),
					new SmoothXyGraphTheme()
				);
			}
		}

		// pairwise tests superimposed and put to graph
//		if (false)
		{
			for (var type : MocapType.values()) {
				var data = loadData(dataFileNames(type), type.dataClass());
				var correlationData = PairwiseCorrelation.compute(data, QUOTIENT);
				for (var category : data.getCategories()) {
					var chart = PairwiseCorrelation.showCategory(correlationData.get(category), QUOTIENT, Optional.of(category)) //ChartFactory.createScatterPlot();
				}
				Utils.writeXYGraphAsPdf(
					PairwiseCorrelation.showCategory(
						PairwiseCorrelation.compute(
							loadData(dataFileNames(type), type.dataClass()),
							QUOTIENT
						),
						QUOTIENT,

					)
					,
					new SmoothXyGraphTheme();
				);
			}
		}

		// kNN tests (to graphs)
		if (false)
		{
			for (var type : MocapType.values()) {
				showKnnLengthGraph.show(
					dataFileNames(type),
					type.dataClass(),
					"output/" + type.adjective() + " kNN graphs"
				);
			}
		}

		// kNN tests (to averages and to file)
		if (false)
		{
			var output = new PrintWriter("output/kNN averages.txt");

			for (var type : MocapType.values()) {
				output.printf("%n%s:%n", type.noun());
				Utils.write(
					Utils.knnPrecisionAverageFormat(KnnPrecision.average(KnnPrecision.compute(loadData(
						dataFileNames(type),
						type.dataClass())))),
					output);
			}
		}
	}

	private static String dataFileNames(MocapType type) {
		return switch (type) {
			case SKELETON -> "data/skeleta/class130-actions-coords_normPOS-fps12.data";
			case HARD_MW -> "data/motion words/hdm05-annotations_specific-segment80_shift16-coords_normPOS-fps12-quantized-pivots-kmedoids-350.data";
			case SOFT_MW -> "data/motion words/kmedoids-350-softassign-D20K6.data";
			case MATCH_N_MW -> "data/motion words/multi-overlay_motion_words.data";
	};}
}
